unit FrmDesign;

{

   COPYRIGHT 2014, 2015 Blue Hell / Jan Punter

  Some parts are copyright :

     Author  : Neugls.
     Website : Http://www.neugls.info
     Email   : NeuglsWorkStudio@gmail.com

   these are marked with (*Neugls*)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    Oogstplein 6
    7545 HP Enschede
    the Netherlands
    http://www.iaf.nl/Users/BlueHell/
    http://bluehell.electro-music.com/
    j_dot.punter2@t2iaf_dot.nl

  All rights attributed to Blue Hell are owned by Jan Punter.
}

interface

uses

  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, ToolWin, StdCtrls, ComCtrls,
  ExtCtrls, Menus, ImgList, Vcl.Buttons, IniFiles, System.UITypes,

  JvDesignSurface, JvDesignUtils, JvExControls, JvInspector, JvResources,

  KnobsUtils, Knobs2013, KnobsConversions, KnobsJittik,

  Globals;

type

  TWeatherRec = record
    Direction     : Integer;  // Wind direction 0 .. 359 -> 0 .. 1
    Speed         : Integer;  // Wind speed
    Temperature   : Integer;
    Sunshine      : Integer;  // 0 - 100
    Precipitation : Integer;
    BaroPressure  : Integer;
    Sight         : Integer;
    Humidity      : Integer;  // 0 - 100
  end;

  TWeatherData = class
  private
    FReadFilename     : string;
    FWriteFilename    : string;
    FRawData          : TStringList;
    FData             : array of TWeatherRec;
    FMinSpeed         : Integer;
    FMaxSpeed         : Integer;
    FMinTemperature   : Integer;
    FMaxTemperature   : Integer;
    FMinPrecipitation : Integer;
    FMaxPrecipitation : Integer;
    FMinBaroPressure  : Integer;
    FMaxBaroPressure  : Integer;
    FMinSight         : Integer;
    FMaxSight         : Integer;
    FMinHumidity      : Integer;
    FMaxHumidity      : Integer;
  public
    constructor Create( const aFilename: string);
    destructor  Destroy;                                   override;
    procedure   Calculate;
    procedure   Write( const aFilename: string);
  end;

  TMainForm = class( TForm)
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    ImageList: TImageList;
    MainMenu: TMainMenu;
    MenuItemFile: TMenuItem;
    MenuItemOpen: TMenuItem;
    Save1: TMenuItem;
    MenuItemNew: TMenuItem;
    MenuItemSep1: TMenuItem;
    MenuItemExit: TMenuItem;
    N1: TMenuItem;
    MenuItemDesigenActive: TMenuItem;
    PageControlMain: TPageControl;
    TabSheetModuleDesign: TTabSheet;
    ToolBar: TToolBar;
    but_select: TToolButton;
    but_module: TToolButton;
    but_led: TToolButton;
    but_selector: TToolButton;
    but_knob: TToolButton;
    but_smallknob: TToolButton;
    but_noknob: TToolButton;
    but_display: TToolButton;
    but_fileselector: TToolButton;
    but_indicator: TToolButton;
    but_indicatorbar: TToolButton;
    but_indicatortext: TToolButton;
    but_dataviewer: TToolButton;
    but_input: TToolButton;
    but_output: TToolButton;
    but_textlabel: TToolButton;
    but_editlabel: TToolButton;
    but_box: TToolButton;
    but_image: TToolButton;
    JvDesignScrollBox: TJvDesignScrollBox;
    JvDesignPanel: TJvDesignPanel;
    JvInspector: TJvInspector;
    Splitter: TSplitter;
    TabSheetModuleContents: TTabSheet;
    TabSheetScripting: TTabSheet;
    Jittik: TKnobsJittik;
    PanelScriptingCommand: TPanel;
    PanelScriptingOutput: TPanel;
    Label1: TLabel;
    MemoScriptOutput: TMemo;
    ComboBoxScriptCommand: TComboBox;
    BitBtnClearDisplay: TBitBtn;
    BitBtnBreak: TBitBtn;
    BitBtnReset: TBitBtn;
    LabelFileInfo: TLabel;
    TimerTest: TTimer;
    Label2: TLabel;
    BitBtn1: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure PaletteButtonClick(Sender: TObject);
    procedure MenuItemNewClick(Sender: TObject);
    procedure JvDesignPanelGetAddClass(Sender: TObject; var ioClass: String);
    procedure MenuItemExitClick(Sender: TObject);
    procedure JvDesignPanelSelectionChange(Sender: TObject);
    procedure MenuItemDesigenActiveClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure JittikBye(aSender: TObject);
    procedure ComboBoxScriptCommandKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure BitBtnClearDisplayClick(Sender: TObject);
    procedure JittikLog(aSender: TObject; const aMsg: string);
    procedure JittikReset(Sender: TObject);
    procedure JittikSay(aSender: TObject; const aMsg: string);
    function JittikUnknownSymbol(aSender: TObject; const aToken: string): Boolean;
    procedure BitBtnBreakClick(Sender: TObject);
    procedure JittikBreak(aSender: TObject; const aMsg: string);
    procedure BitBtnResetClick(Sender: TObject);
    procedure JittikClearScreen(Sender: TObject);
    procedure PageControlMainChange(Sender: TObject);
    procedure JittikFileEnter(aSender: TObject; const aFileName: string);
    procedure JittikFileLeave(aSender: TObject; const aFileName: string);
    procedure JittikFileResume(aSender: TObject; const aFileName: string);
    procedure JittikLine(aSender: TObject; aLineNr, aTotalCount: Integer);
    procedure JittikDataOut(aSender: TObject; const aChannel: string; const aData: TVariants);
    procedure TimerTestTimer(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    FDesignClass     : string;
    FStickyMode      : Boolean;
    FCommands        : TRecentStrings;
  private
    FShowFileEnters  : Boolean;
    FShowFileLeaves  : Boolean;
    FShowFileResumes : Boolean;
    FTimeHandler     : TSymbol;
    FTimeStack       : TJittikStack;
  private
    procedure   LoadIni;
    procedure   SaveIni;
    procedure   DoJittikReset;
    procedure   LeaseTimeHandler;
    procedure   ReleaseTimeHandler;
    procedure   CallTimeHandler;
    procedure   ProcessCommand( const aCmd: string);
    procedure   Display( const aMsg: string);
    procedure   DisplayFmt( const aFmt: string; const anArgs: array of const);
    procedure   ClearDisplay;
    procedure   HandleUnknownData( const aChannel: string; const aData: TVariants);
    procedure   HandleScaleData( const aData: TVariants);
    procedure   HandleMidiData( const aData: TVariants);
    procedure   HandleForthData( const aChannel: string; const aData: TVariants);
  private
//    procedure   CalcMyLife;
//    procedure   Benchmark;
  end;

var

  MainForm: TMainForm;



implementation


uses

  JvDesignImp;

// User area

{ ========
  TWeatherRec = record
    Direction     : Integer;  // Wind direction 0 .. 359 -> 0 .. 1
    Speed         : Integer;  // Wind speed
    Temperature   : Integer;
    Sunshine      : Integer;  // 0 - 100
    Precipitation : Integer;
    BaroPressure  : Integer;
    Sight         : Integer;
    Humidity      : Integer;  // 0 - 100
  end;

  TWeatherData = class
  private
    FReadFilename     : string;
    FWriteFilename    : string;
    FRawData          : TStringList;
    FData             : array of TWeatherRec;
    FMinSpeed         : Integer;
    FMaxSpeed         : Integer;
    FMinTemperature   : Integer;
    FMaxTemperature   : Integer;
    FMinPrecipitation : Integer;
    FMaxPrecipitation : Integer;
    FMinBarePressure  : Integer;
    FMaxBaroPresssure : Integer;
    FMinSight         : Integer;
    FMaxSight         : Integer;
    FMinHumidity      : Integer;
    FMaxHumidity      : Integer;
  public
}

    constructor TWeatherData.Create( const aFilename: string);
    var
      i : Integer;
    begin
      inherited Create;
      FReadFilename := aFilename;
      FRawData := TStringList.Create;
      if FileExists( FReadFilename)
      then begin
        FRawData.LoadFromFile( FReadFilename);
        i := 0;
        while i < FRawData.Count
        do begin
          if Pos( '#', FRawData[ i]) > 0
          then FRawData.Delete( i)
          else Inc( i);
        end;
      end;
    end;

    destructor  TWeatherData.Destroy; // override;
    begin
      FreeAndNil( FRawData);
      inherited;
    end;

    procedure   TWeatherData.Calculate;
    var
      aLine  : string;
      aParts : TStringList;
      i      : Integer;
      p      : Integer;
    begin
      FMinSpeed         :=   MaxInt;
      FMaxSpeed         := - MaxInt;
      FMinTemperature   :=   MaxInt;
      FMaxTemperature   := - MaxInt;
      FMinPrecipitation :=   MaxInt;
      FMaxPrecipitation := - MaxInt;
      FMinBaroPressure  :=   MaxInt;
      FMaxBaroPressure  := - MaxInt;
      FMinSight         :=   MaxInt;
      FMaxSight         := - MaxInt;
      FMinHumidity      :=   MaxInt;
      FMaxHumidity      := - MaxInt;
      SetLength( FData, FRawData.Count);

      p := 0;
      for aLine in FRawData
      do begin
        aParts := Explode( aLine, ',');
        try
          for i := 0 to aParts.Count - 1
          do aParts[ i] := Trim( aParts[ i]);

          with FData[ p]
          do begin
            Direction := StrToIntDef( aParts[ 2], 0) mod 360;

            Speed := StrToIntDef( aParts[ 3], 0);
            if Speed < FMinSpeed then FMinSpeed := Speed;
            if Speed > FMaxSpeed then FMaxSpeed := Speed;

            Temperature := StrToIntDef( aParts[ 4], 0);
            if Temperature < FMinTemperature then FMinTemperature := Temperature;
            if Temperature > FMaxTemperature then FMaxTemperature := Temperature;

            Sunshine := StrToIntDef( aParts[ 5], 0);

            Precipitation := StrToIntDef( aParts[ 6], 0);
            if Precipitation < 0
            then Precipitation := 0;
            if Precipitation < FMinPrecipitation then FMinPrecipitation := Precipitation;
            if Precipitation > FMaxPrecipitation then FMaxPrecipitation := Precipitation;

            BaroPressure := StrToIntDef( aParts[ 7], 0);
            if BaroPressure < FMinBaroPressure then FMinBaroPressure := BaroPressure;
            if BaroPressure > FMaxBaroPressure then FMaxBaroPressure := BaroPressure;

            Sight := StrToIntDef( aParts[ 8], 0);
            if Sight < FMinSight then FMinSight := Sight;
            if Sight > FMaxSight then FMaxSight := Sight;

            Humidity := StrToIntDef( aParts[ 9], 0);
            if Humidity < FMinHumidity then FMinHumidity := Humidity;
            if Humidity > FMaxHumidity then FMaxHumidity := Humidity;
          end;
        finally
          aParts.Free;
        end;
        Inc( p);
      end;
    end;

    procedure   TWeatherData.Write( const aFilename: string);

      function Scale( const aName: string; aValue, aMin, aMax: Integer): Double;
      begin
        Result := ( aValue - aMin) / ( aMax - aMin);
      end;

    var
      i     : Integer;
      aData : TStringList;
    begin
      FWriteFilename := aFilename;
      aData := TStringList.Create;
      try
        aData.Add( Format( '# MinSpeed         = %d', [ FMinSpeed         ]));
        aData.Add( Format( '# MaxSpeed         = %d', [ FMaxSpeed         ]));
        aData.Add( Format( '# MinTemperature   = %d', [ FMinTemperature   ]));
        aData.Add( Format( '# MaxTemperature   = %d', [ FMaxTemperature   ]));
        aData.Add( Format( '# MinPrecipitation = %d', [ FMinPrecipitation ]));
        aData.Add( Format( '# MaxPrecipitation = %d', [ FMaxPrecipitation ]));
        aData.Add( Format( '# MinBaroPressure  = %d', [ FMinBaroPressure  ]));
        aData.Add( Format( '# MaxBaroPressure  = %d', [ FMaxBaroPressure  ]));
        aData.Add( Format( '# MinSight         = %d', [ FMinSight         ]));
        aData.Add( Format( '# MaxSight         = %d', [ FMaxSight         ]));
        aData.Add( Format( '# MinHumidity      = %d', [ FMinHumidity      ]));
        aData.Add( Format( '# MaxHumidity      = %d', [ FMaxHumidity      ]));
        for i := 0 to Length( FData) - 1
        do begin
          with FData[ i]
          do begin
            aData.Add(
              Format( '%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f',
                [
                  Direction / 360,
                  Scale( 'speed', Speed, FMinSpeed, FMaxSpeed),
                  Scale( 'temperature', Temperature, FMinTemperature, FMaxTemperature),
                  Sunshine / 100,
                  Scale( 'precipitation', Precipitation, FMinPrecipitation, FMaxPrecipitation),
                  Scale( 'baropressure', BaroPressure, FMinBaroPressure, FMaxBAroPressure),
                  Scale( 'sight', Sight, FMinSight, FMaxSight),
                  Scale( 'humidity', Humidity, FMinHumidity, FMaxHumidity)
                ]
              )
            );
          end;
        end;
        aData.SaveToFile( FWriteFilename);
      finally
        aData.Free;
      end;
    end;


  function Color2RGB( aColor: TColor):Integer;
  (*Neugls*)
  begin
    Result := 0;
    Result := Result or ((( aColor shr  0) and $ff) shl 16);
    Result := Result or ((( aColor shr  8) and $ff) shl  8);
    Result := Result or ((( aColor shr 16) and $ff) shl  0);
  end;


type

  TJvInspectorColorItem = class( TJvCustomInspectorItem)
  (*Neugls*)
  protected
    function    GetColorName( const Color: TColor): string;
    procedure   PaintValue( const Color: TColor; const ColorName: string;const ACanvas: TCanvas; const ARect: TRect);
  protected
    procedure   SetFlags( const Value: TInspectorItemFlags);                                                   override;
    procedure   SetRects( const RectKind: TInspectorPaintRect; Value: TRect);                                  override;
    procedure   ButtonClick( Sender: TObject);                                                                 override;
    function    GetDisplayValue: string;                                                                       override;
  public
    constructor Create( const AParent: TJvCustomInspectorItem; const AData: TJvCustomInspectorData);           override;
    procedure   DrawValue( const ACanvas: TCanvas);                                                            override;
  public
    class procedure RegisterAsDefaultItem;
    class procedure UnregisterAsDefaultItem;
  end;

  TJvInspectorControlTypeItem = class( TJvCustomInspectorItem)
  protected
    procedure   SetFlags( const Value: TInspectorItemFlags);                                                   override;
  public
    constructor Create( const AParent: TJvCustomInspectorItem; const AData: TJvCustomInspectorData);           override;
  public
    class procedure RegisterAsDefaultItem;
    class procedure UnregisterAsDefaultItem;
  end;


{ ========
  TJvInspectorColorItem = class( TJvCustomInspectorItem)
  protected
}

    function    TJvInspectorColorItem.GetColorName( const Color: TColor): string;
    (*Neugls*)
    var
      RGBColor:Integer;
    begin
      RGBColor := Color2RGB( Color);
      Result   := '$'+ IntToHex( RGBColor, 6);
    end;

    procedure   TJvInspectorColorItem.PaintValue( const Color: TColor; const ColorName: string;const ACanvas: TCanvas; const ARect: TRect);
    (*Neugls*)
    var
      TH      : Integer;
      BoxRect : TRect;
      bc      : TColor;
      pc      : TColor;
      txtRect : TRect;
    begin
      TH             := Rects[ iprValue].Bottom - Rects[ iprValue].Top - 2;
      BoxRect.Left   := ARect.Left + ( ARect.Bottom - ARect.Top - TH) div 2;
      BoxRect.Top    := ARect.Top  + BoxRect.Left - ARect.Left;
      BoxRect.Right  := BoxRect.Left + TH;
      BoxRect.Bottom := BoxRect.Top  + TH;
      with ACanvas do
      begin
        if Color <> clNone then
        begin
          bc := Brush.Color;
          pc := Pen.Color;
          try
            Brush.Color := Color;
            Pen.Color   := clBlack;
            Rectangle(BoxRect);
          finally
            Pen.Color := pc;
            Brush.Color := bc;
          end;
        end;
       txtRect := ARect;
       txtRect.Left := txtRect.Left + (txtRect.Bottom-txtRect.Top)+ 1;
       TextRect(txtRect, txtRect.Left, BoxRect.Top, ColorName);
      end;
    end;

//  protected

    procedure   TJvInspectorColorItem.SetFlags( const Value: TInspectorItemFlags); // override;
    (*Neugls*)
    begin
      inherited SetFlags( Value + [ iifEditButton, iifEditFixed]);
    end;

    procedure   TJvInspectorColorItem.SetRects( const RectKind: TInspectorPaintRect; Value: TRect); // override;
    (*Neugls*)
    begin
      if RectKind = iprValue
      then Value.Left := Value.Left + (Value.Bottom - Value.Top) + 2;
      inherited SetRects( RectKind, Value);
    end;

    procedure   TJvInspectorColorItem.ButtonClick( Sender: TObject); // override;
    (*Neugls*)
    var
      ColorDlg: TColorDialog;
    begin
      ColorDlg := TColorDialog.Create( nil);
      try
        if ColorDlg.Execute
        then begin
       // DisplayValue := NameForColor( FColor);
          Data.AsOrdinal := ColorDlg.Color;
        end;
      finally
        ColorDlg.Free;
      end;
      inherited;
    end;

    function    TJvInspectorColorItem.GetDisplayValue: string; // override;
    (*Neugls*)
    begin
      Result := GetColorName( Data.AsOrdinal);
    end;

//  public

    constructor TJvInspectorColorItem.Create( const AParent: TJvCustomInspectorItem; const AData: TJvCustomInspectorData); // override;
    (*Neugls*)
    begin
      inherited Create( AParent, AData);
      Flags := [ iifVisible, iifEditButton, iifEditFixed];
    end;

    procedure   TJvInspectorColorItem.DrawValue( const ACanvas: TCanvas); // override;
    (*Neugls*)
    var
      Color     : TColor;
      S         : string;
      ARect     : TRect;
      SafeColor : TColor;
    begin
      Color := clNone;

      if Data = nil
      then S := RsJvInspItemUnInitialized
      else begin
        try
          if not Data.IsInitialized
          then S := RsJvInspItemUnInitialized
          else if not Data.HasValue
          then S := RsJvInspItemNoValue
          else if not Data.IsAssigned
          then S := RsJvInspItemUnassigned
          else begin
            S     := DisplayValue;
            Color := Data.AsOrdinal;
          end;
        except
          on E: Exception
          do S := RsJvInspItemValueException + E.ClassName + ': ' + E.Message;
        end;

        ARect     := Rects[ iprValueArea];
        SafeColor := ACanvas.Brush.Color;

        if Editing
        then ACanvas.Brush.Color := clWindow;

        try
          ACanvas.FillRect( ARect);
          PaintValue( Color, S, ACanvas, ARect);
          if Editing
          then DrawEditor( ACanvas);
        finally
          if Editing
          then ACanvas.Brush.Color := SafeColor;
        end;
      end;
    end;

//  public

    class procedure TJvInspectorColorItem.RegisterAsDefaultItem;
    (*Neugls*)
    begin
      with TJvCustomInspectorData.ItemRegister
      do begin
        if IndexOf( Self) = -1
        then Add( TJvInspectorTypeInfoRegItem.Create( Self, TypeInfo( TColor)));
      end;
    end;

    class procedure TJvInspectorColorItem.UnregisterAsDefaultItem;
    (*Neugls*)
    begin
      TJvCustomInspectorData.ItemRegister.Delete( Self);
    end;


{ ========
  TJvInspectorControlTypeItem = class( TJvCustomInspectorItem)
  protected
}

    procedure   TJvInspectorControlTypeItem.SetFlags( const Value: TInspectorItemFlags); // override;
    begin
      inherited SetFlags( Value + [ iifEditButton, iifEditFixed, iifValueList]);
    end;

//  public

    constructor TJvInspectorControlTypeItem.Create( const AParent: TJvCustomInspectorItem; const AData: TJvCustomInspectorData); // override;
    begin
      inherited Create( AParent, AData);
      Flags := [ iifVisible, iifEditButton, iifEditFixed, iifValueList];
    end;

//  public

    class procedure TJvInspectorControlTypeItem.RegisterAsDefaultItem;
    begin
      with TJvCustomInspectorData.ItemRegister
      do begin
        if IndexOf( Self) = -1
        then Add( TJvInspectorTypeInfoRegItem.Create( Self, TypeInfo( TColor)));
      end;
    end;

    class procedure TJvInspectorControlTypeItem.UnregisterAsDefaultItem;
    begin
      TJvCustomInspectorData.ItemRegister.Delete( Self);
    end;


//  private

const

  sdfs = 'Defaults';
  sdws = 'window';
  hist = 'CmdHistory';

    procedure   TMainForm.LoadIni;
    var
      anIniFile    : TMemIniFile;
      L, T, W, H   : Integer;
      WinState     : TWindowState;
    begin
      anIniFile := TMemIniFile.Create( IniFileName);
      with anIniFile
      do begin
        try
          FCommands.LoadFromIni( anIniFile, hist);
          ComboBoxScriptCommand.Items.Assign( FCommands);
          if FCommands.Count = 0
          then ComboBoxScriptCommand.Text := 'help // put the cursor on this line and press enter'
          else ComboBoxScriptCommand.Text := '';

          L        :=                ReadInteger( sdws, 'Left'       ,          Left         );
          T        :=                ReadInteger( sdws, 'Top'        ,          Top          );
          W        :=                ReadInteger( sdws, 'Width'      ,          Width        );
          H        :=                ReadInteger( sdws, 'Height'     ,          Height       );
          WinState := TWindowState(  ReadInteger( sdws, 'WindowState', Integer( WindowState)));

          if ( L < 0) or ( L > Screen.Width  - 20) then L := 0;
          if ( T < 0) or ( T > Screen.Height - 20) then T := 0;
          if W > Screen.Width                      then W := Screen.Width  - 20;
          if H > Screen.Height                     then H := Screen.Height - 20;

          SetBounds( L, T, W, H);
          if WinState = wsMaximized
          then WindowState := wsMaximized;
        finally
          Free;
        end;
      end;
    end;

    procedure   TMainForm.SaveIni;
    var
      anIniFile : TMemIniFile;
    begin
      anIniFile := TMemIniFile.Create( IniFileName);
      with anIniFile
      do begin
        try
          FCommands.SaveToIni( anIniFile, hist);
          EraseSection( sdfs);
          if WindowState = wsNormal
          then begin
            WriteInteger( sdws, 'Left'  , Left  );
            WriteInteger( sdws, 'Top'   , Top   );
            WriteInteger( sdws, 'Width' , Width );
            WriteInteger( sdws, 'Height', Height);
          end;
          WriteInteger( sdws, 'WindowState', Integer( WindowState));
        finally
          UpdateFile;
          Free;
        end;
      end;
    end;

    procedure   TMainForm.DoJittikReset;
    begin
      BitBtnBreak.Enabled := True;
      try
        TimerTest.Enabled := False;
        ReleaseTimeHandler;
        if Jittik.Reset
        then begin
          LeaseTimeHandler;
          TimerTest.Enabled := True;
        end;
      finally
        BitBtnBreak.Enabled := False;
      end;
    end;

    procedure   TMainForm.LeaseTimeHandler;
    begin
      if Assigned( Jittik)
      then begin
        FTimeStack   := Jittik.LeaseStackImage;
        FTimeHandler := Jittik.LeaseSymbol    ( 'time_handler');
      end;
    end;

    procedure   TMainForm.ReleaseTimeHandler;
    begin
      if Assigned( Jittik)
      then begin
        Jittik.ReleaseSymbol    ( FTimeHandler);
        Jittik.ReleaseStackImage( FTimeStack  );
      end;
    end;

    procedure   TMainForm.CallTimeHandler;
    begin
      FTimeStack.PushValues( TVariants.Create( Now, 1));
      Jittik.AcceptWithStack( FTimeHandler, FTimeStack);
      FTimeStack.Clear;
    end;

    procedure   TMainForm.ProcessCommand( const aCmd: string);
    begin
      if not Jittik.Interpreting
      then begin
        BitBtnBreak.Enabled := True;
        try
          Jittik.AcceptCode( aCmd);
        finally
          BitBtnBreak.Enabled := False;
          if Assigned( FCommands)
          then begin
            FCommands.SetMostRecent( aCmd);
            ComboBoxScriptCommand.Items.Assign( FCommands);
          end;
          ComboBoxScriptCommand.Text := '';
        end;
      end;
    end;

    procedure   TMainForm.Display( const aMsg: string);
    begin
      MemoScriptOutput.Lines.Add( aMsg);
    end;

    procedure   TMainForm.DisplayFmt( const aFmt: string; const anArgs: array of const);
    begin
      Display( Format( aFmt, anArgs, AppLocale));
    end;

    procedure   TMainForm.ClearDisplay;
    begin
      MemoScriptOutput.Clear;
    end;

    procedure   TMainForm.HandleUnknownData( const aChannel: string; const aData: TVariants);
    var
      S : string;
      i : Integer;
    begin
      S := '';
      for i := 0 to Length( aData) - 1
      do begin
        if S = ''
        then S :=     Format(  '%s', [ aData[ i]])
        else S := S + Format( ' %s', [ aData[ i]]);
      end;
      DisplayFmt( 'unhandled jittik request: "%s" %s', [ aChannel, S]);
    end;

    procedure   TMainForm.HandleMidiData( const aData: TVariants);
    var
      S : string;
      i : Integer;
    begin
      S := '';
      for i := 0 to Length( aData) - 1
      do begin
        if S = ''
        then S :=     Format(  '%s', [ aData[ i]])
        else S := S + Format( ' %s', [ aData[ i]]);
      end;
      DisplayFmt( 'midi request ( %s )', [ S]);
    end;

    procedure   TMainForm.HandleScaleData( const aData: TVariants);
    var
      S : string;
      i : Integer;
    begin
      S := '';
      for i := 0 to Length( aData) - 1
      do begin
        if S = ''
        then S :=     Format(  '%s', [ aData[ i]])
        else S := S + Format( ' %s', [ aData[ i]]);
      end;
      DisplayFmt( 'scale request ( %s )', [ S]);
    end;

    procedure   TMainForm.HandleForthData( const aChannel: string; const aData: TVariants);
    begin
      if SameText( aChannel, 'midi')
      then HandleMidiData( aData)
      else if SameText( aChannel, 'scale')
      then HandleScaleData( aData)
      else HandleUnknownData( aChannel, aData);
    end;

//  private

//    procedure   TMainForm.Benchmark;
//    const
//      Iterations = 1000000000;
//    var
//      aTime : TDateTime;
//      i     : Cardinal;
//      V     : Double;
//      A     : Integer;
//      B     : Integer;
//      C     : Integer;
//    begin
//      V := 1.000000001;
//      aTime := Now;
//      for i := 1 to Iterations
//      do V := V / V;
//      aTime := Now - aTime;
//      DisplayFmt( '/ time : %.6f', [ aTime * 100000]);
//      V := 1;
//      aTime := Now;
//      for i := 1 to Iterations
//      do V := V * V;
//      aTime := Now - aTime;
//      DisplayFmt( '* time : %.6f', [ aTime * 100000]);
//      A := 10333;
//      B := 3;
//      aTime := Now;
//      for i := 1 to Iterations
//      do C := ( A * B) shr 8;
//      aTime := Now - aTime;
//      DisplayFmt( 'int * time : %.6f', [ aTime * 100000]);
//      A := 10333;
//      B := 3;
//      aTime := Now;
//      for i := 1 to Iterations
//      do C := ( A div B) shl 8;
//      aTime := Now - aTime;
//      DisplayFmt( 'int / time : %.6f', [ aTime * 100000]);
//    end;

//    procedure   TMainForm.CalcMyLife;
//    var
//      aData: TWeatherData;
//    begin
//      aData := TWeatherData.Create( 'mylife.txt');
//      try
//        aData.Calculate;
//        aData.Write( 'mylife_out.txt');
//      finally
//        aData.Free;
//      end;
//    end;

// Delphi area

{$R *.dfm}

procedure TMainForm.TimerTestTimer(Sender: TObject);
//var
//  aData : TVariants;
begin
//  SetLength( aData, 1);
//  aData[ 0] := Now;
//  Jittik.AcceptAction( 'time_handler', aData);
  CallTimeHandler;
end;

procedure TMainForm.BitBtn1Click(Sender: TObject);
begin
//  CalcMyLife;
//  Benchmark;
end;

procedure TMainForm.BitBtnBreakClick(Sender: TObject);
begin
  Jittik.DoBreak;
end;

procedure TMainForm.BitBtnClearDisplayClick(Sender: TObject);
begin
  ClearDisplay;
end;

procedure TMainForm.BitBtnResetClick(Sender: TObject);
begin
  DoJittikReset;
end;

procedure TMainForm.ComboBoxScriptCommandKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  S : string;
begin
  if Shift = []
  then begin
    if Key = VK_RETURN
    then begin
      S := ComboBoxScriptCommand.Text;
      DisplayFmt( '>> %s', [ S]);
      ProcessCommand( S);
    end;
  end;
end;

procedure TMainForm.MenuItemExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.MenuItemDesigenActiveClick(Sender: TObject);
begin
  JvDesignPanel.Active := not JvDesignPanel.Active;
  MenuItemDesigenActive.Checked := JvDesignPanel.Active;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ReleaseTimeHandler;
  SaveIni;
  FreeAndNil( FCommands);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FCommands := TRecentStrings.Create( 100);
  OpenDialog.InitialDir := ExtractFilePath(Application.ExeName);
  SaveDialog.InitialDir := OpenDialog.InitialDir;
  JvDesignPanel.Surface.MessengerClass := TJvDesignWinControlHookMessenger;
// JvDesignPanel.Surface.MessengerClass := TJvDesignDesignerMessenger;
  JvDesignPanel.Surface.Active  := True;
  MenuItemDesigenActive.Checked := JvDesignPanel.Surface.Active;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  PageControlMain.ActivePage := TabSheetScripting;
  ComboBoxScriptCommand.SetFocus;
  Jittik.StartupFile := JittikFileName;
  ClearDisplay;
  ComboBoxScriptCommand.Text := '';
  BitBtnBreak.Enabled := False;
  LoadIni;
  DoJittikReset;
end;

procedure TMainForm.JittikBreak(aSender: TObject; const aMsg: string);
begin
  DisplayFmt( 'scripting break requested: %s', [ aMsg]);
end;

procedure TMainForm.JittikBye(aSender: TObject);
begin
  Display( 'scripting "bye" executed, shutting down ...');
  Close;
end;

procedure TMainForm.JittikClearScreen(Sender: TObject);
begin
  ClearDisplay;
end;

procedure TMainForm.JittikDataOut(aSender: TObject; const aChannel: string; const aData: TVariants);
begin
  HandleForthData( aChannel, aData);
end;

procedure TMainForm.JittikFileEnter(aSender: TObject; const aFileName: string);
begin
  if FShowFileEnters
  then DisplayFmt( 'entering: %s', [ aFileName]);
end;

procedure TMainForm.JittikFileLeave(aSender: TObject; const aFileName: string);
begin
  if FShowFileLEaves
  then DisplayFmt( 'leaving : %s', [ aFileName]);
end;

procedure TMainForm.JittikFileResume(aSender: TObject; const aFileName: string);
begin
  if FShowFileResumes
  then DisplayFmt( 'resuming: %s', [ aFileName]);
end;

procedure TMainForm.JittikLine(aSender: TObject; aLineNr, aTotalCount: Integer);
begin
  LabelFileInfo.Caption := Format( 'lines: last file %d - total %d', [ aLineNr, aTotalCount])
end;

procedure TMainForm.JittikLog(aSender: TObject; const aMsg: string);
begin
  DisplayFmt( '==> %s', [ aMsg]);
end;

procedure TMainForm.JittikReset(Sender: TObject);
begin
  Display( 'performed system reset');
end;

procedure TMainForm.JittikSay(aSender: TObject; const aMsg: string);
begin
  Display( aMsg);
end;

function TMainForm.JittikUnknownSymbol(aSender: TObject; const aToken: string): Boolean;
begin
  // This is a hook function, when the system does not know the word in aToken
  // it will call into here, when the problem can be solved here make this function
  // return true. When false is returned an error message will be shown on the
  // unknown token (through the ForthSay callback(.
  Result := False;
end;

procedure TMainForm.JvDesignPanelGetAddClass(Sender: TObject;
  var ioClass: String);
begin
  ioClass := FDesignClass;
  if not FStickyMode
  then begin
    FDesignClass := '';
    but_select.Down := True;
  end;
end;

procedure TMainForm.PageControlMainChange(Sender: TObject);
begin
  if PageControlMain.ActivePage = TabSheetScripting
  then ComboBoxScriptCommand.SetFocus;
end;

procedure TMainForm.PaletteButtonClick(Sender: TObject);
const
  cClasses: array[ 0 .. 18] of string =
  (
    ''                   ,
    'TKnobsModule'       ,
    'TKnobsLed'          ,
    'TKnobsSelector'     ,
    'TKnobsKnob'         ,
    'TKnobsSmallKnob'    ,
    'TKnobsNoKnob'       ,
    'TKnobsDisplay'      ,
    'TKnobsFileSelector' ,
    'TKnobsIndicator'    ,
    'TKnobsIndicatorBar' ,
    'TKnobsIndicatorText',
    'TKnobsDataViewer'   ,
    'TKnobsInput'        ,
    'TKnobsOutput'       ,
    'TKnobsTextLabel'    ,
    'TKnobsEditLabel'    ,
    'TKnobsBox'          ,
    'TImage'
  );
begin
  FStickyMode  := GetKeyState( VK_SHIFT) < 0;
  FDesignClass := cClasses[TControl( Sender).Tag];
end;

procedure TMainForm.MenuItemNewClick(Sender: TObject);
begin
  JvDesignPanel.Clear;
end;

procedure TMainForm.OpenButtonClick(Sender: TObject);
begin
  if OpenDialog.Execute
  then JvDesignPanel.LoadFromFile( OpenDialog.Filename);
end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
begin
  if SaveDialog.Execute
  then JvDesignPanel.SaveToFile( SaveDialog.Filename);
end;

procedure TMainForm.JvDesignPanelSelectionChange(Sender: TObject);
begin
  if Length( JvDesignPanel.Surface.Selected) > 0
  then begin
    // JvInspector.AddComponent( JvDesignPanel.Surface.Selected[ 0]);
    JvInspector.InspectObject := JvDesignPanel.Surface.Selected[ 0];
  end
  else JvInspector.InspectObject := nil;
end;


initialization


  RegisterClass( TKnobsModule       );
  RegisterClass( TKnobsLed          );
  RegisterClass( TKnobsSelector     );
  RegisterClass( TKnobsKnob         );
  RegisterClass( TKnobsSmallKnob    );
  RegisterClass( TKnobsNoKnob       );
  RegisterClass( TKnobsDisplay      );
  RegisterClass( TKnobsFileSelector );
  RegisterClass( TKnobsIndicator    );
  RegisterClass( TKnobsIndicatorBar );
  RegisterClass( TKnobsIndicatorText);
  RegisterClass( TKnobsDataViewer   );
  RegisterClass( TKnobsInput        );
  RegisterClass( TKnobsOutput       );
  RegisterClass( TKnobsTextLabel    );
  RegisterClass( TKnobsEditLabel    );
  RegisterClass( TKnobsBox          );
  RegisterClass( TImage             );

  TJvInspectorColorItem      .RegisterAsDefaultItem;
  TJvInspectorControlTypeItem.RegisterAsDefaultItem;

end.
